﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Caching;
using Microsoft.ApplicationServer.Caching;

namespace CommunityDaysDemo.AppFabric
{
    public class AppFabricCache : ObjectCache
    {
        private string cacheName;

        public AppFabricCache(string cacheName)
        {
            this.cacheName = cacheName;
        }

        public override object AddOrGetExisting(string key, object value, CacheItemPolicy policy, string regionName = null)
        {
            var res = this.Get(key, regionName);
            if (res == null)
                this.Set(key, value, policy, regionName);

            return res;
        }

        public override CacheItem AddOrGetExisting(CacheItem value, CacheItemPolicy policy)
        {
            var res = this.AddOrGetExisting(value.Key, value.Value, policy, value.RegionName);
            return new CacheItem(value.Key, res, value.RegionName);
        }

        public override object AddOrGetExisting(string key, object value, DateTimeOffset absoluteExpiration, string regionName = null)
        {
            return this.AddOrGetExisting(key, value, new CacheItemPolicy { AbsoluteExpiration = absoluteExpiration }, regionName);
        }

        public override bool Contains(string key, string regionName = null)
        {
            return this.Get(key, regionName) != null;
        }

        public override CacheEntryChangeMonitor CreateCacheEntryChangeMonitor(IEnumerable<string> keys, string regionName = null)
        {
            throw new NotImplementedException();
        }

        public override DefaultCacheCapabilities DefaultCacheCapabilities
        {
            get { return DefaultCacheCapabilities.AbsoluteExpirations | System.Runtime.Caching.DefaultCacheCapabilities.OutOfProcessProvider; }
        }

        public override object Get(string key, string regionName = null)
        {
            // configurazione su Web.config
            using (var factory = new DataCacheFactory())
            {
                var cache = factory.GetCache(cacheName);

                return regionName == null ? cache.Get(key) : cache.Get(key, regionName);
            }
        }

        public override CacheItem GetCacheItem(string key, string regionName = null)
        {
            var res = this.Get(key, regionName);
            if (res == null)
                return null;

            return new CacheItem(key, res, regionName);
        }

        public override long GetCount(string regionName = null)
        {
            throw new NotImplementedException();
        }

        protected override IEnumerator<KeyValuePair<string, object>> GetEnumerator()
        {
            throw new NotImplementedException();
        }

        public override IDictionary<string, object> GetValues(IEnumerable<string> keys, string regionName = null)
        {
            throw new NotImplementedException();
        }

        public override string Name
        {
            get { throw new NotImplementedException(); }
        }

        public override object Remove(string key, string regionName = null)
        {
            var res = regionName == null ? this.Get(key) : this.Get(key, regionName);

            using (var factory = new DataCacheFactory())
            {
                var cache = factory.GetCache(cacheName);
                if (regionName == null)
                    cache.Remove(key);
                else
                    cache.Remove(key, regionName);
            }
            return res;
        }

        public override void Set(string key, object value, CacheItemPolicy policy, string regionName = null)
        {
            using (var factory = new DataCacheFactory())
            {
                var cache = factory.GetCache(cacheName);
                TimeSpan? timeout = this.GetTimeout(policy);
                if (timeout == null)
                {
                    if (regionName == null)
                        cache.Put(key, value);
                    else
                        cache.Put(key, value, regionName);
                }
                else
                {
                    if (regionName == null)
                        cache.Put(key, value, timeout.Value);
                    else
                        cache.Put(key, value, timeout.Value, regionName);
                }
            }
        }

        private TimeSpan? GetTimeout(CacheItemPolicy policy)
        {
            if (policy == null || policy.AbsoluteExpiration == ObjectCache.InfiniteAbsoluteExpiration)
                return null;

            return policy.AbsoluteExpiration.UtcDateTime - DateTime.UtcNow;
        }

        public override void Set(CacheItem item, CacheItemPolicy policy)
        {
            this.Set(item.Key, item.Value, policy);
        }

        public override void Set(string key, object value, DateTimeOffset absoluteExpiration, string regionName = null)
        {
            this.Set(key, value, new CacheItemPolicy { AbsoluteExpiration = absoluteExpiration }, regionName);
        }

        public override object this[string key]
        {
            get
            {
                return this.Get(key);
            }
            set
            {
                this.Set(key, value, null, null);
            }
        }
    }
}
