﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Routing;

namespace CommunityDaysDemo.Utilities.Routing
{
    public static class RouteHelpers
    {
        public static string AppendTrailingSlash(this string url)
        {
            int indexOfQueryString = url.IndexOf("?");
            if (indexOfQueryString != -1)
            {
                url = string.Concat(
                    VirtualPathUtility.AppendTrailingSlash(url.Substring(0, indexOfQueryString)),
                    url.Substring(indexOfQueryString));
            }
            else
            {
                url = VirtualPathUtility.AppendTrailingSlash(url);
            }

            return url;
        }

        public static string GetExtension(this string url)
        {
            int indexOfQueryString = url.IndexOf("?");
            if (indexOfQueryString != -1)
            {
                url = url.Substring(0, indexOfQueryString);
            }

            // url is domain name
            if (url.ToLower().StartsWith("http://") &&
                url.Substring(7).IndexOf("/") == -1)
                return string.Empty;

            return VirtualPathUtility.GetExtension(url);
        }

        public static Route MapSEOPageRoute(this RouteCollection routes, string name, string url, string physicalFile)
        {
            return MapSEOPageRoute(routes, name, url, physicalFile, null, null);
        }

        public static Route MapSEOPageRoute(this RouteCollection routes, string name, string url, string physicalFile, object defaults)
        {
            return MapSEOPageRoute(routes, name, url, physicalFile, defaults, null);
        }

        public static Route MapSEOPageRoute(this RouteCollection routes, string name, string url, string physicalFile, object defaults, object constraints)
        {
            var route = new SEORoute(
                url, new RouteValueDictionary(defaults),
                new RouteValueDictionary(constraints),
                new PageRouteHandler(physicalFile));

            routes.Add(name, route);

            return route;
        }
    }
}